﻿Option Strict On

<DebuggerStepThrough()> _
<DebuggerDisplay("{ToString}")> _
Public Structure Vector2

    Private ReadOnly m_x As Double
    Private ReadOnly m_y As Double

    Private ReadOnly m_LenSq As Double

    Public ReadOnly Property x() As Double
        Get
            Return m_x
        End Get
    End Property

    Public ReadOnly Property y() As Double
        Get
            Return m_y
        End Get
    End Property

    Public Sub New(ByVal v As Vector2)
        Me.New(v.x, v.y)
    End Sub

    Public Sub New(ByVal x As Double, ByVal y As Double)
        m_x = x
        m_y = y
        m_LenSq = x * x + y * y
    End Sub

    Public ReadOnly Property LengthSq() As Double
        Get
            Return m_LenSq
        End Get
    End Property

    Public ReadOnly Property Length() As Double
        Get
            Return Math.Sqrt(m_LenSq)
        End Get
    End Property

    Public ReadOnly Property Normalize() As Vector2
        Get
            Return Me / Me.Length
        End Get
    End Property

    Public Shared Operator +(ByVal a As Vector2, ByVal b As Vector2) As Vector2
        Return New Vector2(a.x + b.x, a.y + b.y)
    End Operator

    Public Shared Operator -(ByVal a As Vector2, ByVal b As Vector2) As Vector2
        Return New Vector2(a.x - b.x, a.y - b.y)
    End Operator

    Public Shared Operator *(ByVal a As Vector2, ByVal b As Vector2) As Double
        Return a.x * b.x + a.y * b.y
    End Operator

    Public Shared Operator *(ByVal v As Vector2, ByVal r As Double) As Vector2
        Return New Vector2(v.x * r, v.y * r)
    End Operator

    Public Shared Operator *(ByVal r As Double, ByVal v As Vector2) As Vector2
        Return v * r
    End Operator

    Public Shared Operator /(ByVal v As Vector2, ByVal r As Double) As Vector2
        Return New Vector2(v.x / r, v.y / r)
    End Operator

    Public Shared Operator =(ByVal a As Vector2, ByVal b As Vector2) As Boolean
        Return a.x = b.x AndAlso a.y = b.y
    End Operator

    Public Shared Operator <>(ByVal a As Vector2, ByVal b As Vector2) As Boolean
        Return Not a = b
    End Operator

    Public Overrides Function ToString() As String
        Return String.Format("({0}|{1})", x, y)
    End Function

End Structure